{
This code is intended to test a median filter on all three axis of ADXL345 Accelerometer. It is by no
means optimized.  It is an 11 point filter so there is an array for each axis and the oldest value is
discarded every loop.  Once the hold arrays are refreshed they are transfered to a temporary array to be
sorted. The center value of the sorted array is then the median value.
The raw axis value are displayed in PST followed by the median values for each axis.
}
CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

        #0,ASC,DESC
        
VAR
  long DisplayStack[50]                                 'Array for Cog to Display data.
  long aX,aY,aZ                                         'Raw ACC Values
  long aXhold[11],aYhold[11],aZhold[11]                 'Arrays for ACC Median Filter.
  long Sort[11]                                         'Sorting Array for finding Median Value.
  long DebugHold[20]                                    'Variables for displaying in P.S.T.
  long Elapsed                                          'Used for Timing Sections of Code.
  long uS                                               'Used to convert Section time to uS.
  long Cog                                              'Cog Tracker
  long LastTime                                         'Used to time main Loop
OBJ

  PST      : "Parallax Serial Terminal"                 'By Jeff Martin, Andy Lindsay, Chip Gracey.
  IMU      : "9DOF PASM"                                'By Jason Dorie.
'*********************************************   
PUB Main

  PST.Start(115_200)                                    'Start Parallax Serial Terminal.
  IMU.Start(4,5) '22,23                                 'Starts IMU(9DOF SparkFun) in a new Cog, sets SCL to 4 and SDA to 5.
  waitcnt(clkfreq*5 + cnt)                              'Pause 5 Seconds for initialization.
  Initiate_Display_Cog
  uS := 80_000_000/1000000
  LastTime := cnt
  Repeat
    Get_ACC
    ACC_Median_Filter
    waitcnt( constant(80_000_000 / 200) + LastTime )
    LastTime += constant(80_000_000 / 200)

'*********************************************     
PUB Get_ACC
'Get raw values from the ACC
  aX := IMU.GetAX - 24                                  '24 is my offset while board is sitting on the table.
  DebugHold[0] := aX 
  aY := IMU.GetAY - 12                                  '12 is my offset while board is sitting on the table.
  DebugHold[1] := aY 
  aZ := IMU.GetAZ - 9                                   '9 is my offset while board is sitting on the table.
  DebugHold[2] := aZ 

'*********************************************  
PUB ACC_Median_Filter | i

  elapsed := -cnt                                       'Used to time all 3 sorts.
'Disgards oldest aX value and reorders hold array.
  aXhold[10] := aXhold[9]
  aXhold[9]  := aXhold[8]
  aXhold[8]  := aXhold[7]
  aXhold[7]  := aXhold[6]
  aXhold[6]  := aXhold[5]
  aXhold[5]  := aXhold[4]
  aXhold[4]  := aXhold[3]
  aXhold[3]  := aXhold[2]
  aXhold[2]  := aXhold[1]
  aXhold[1]  := aXhold[0] 
  aXhold[0]  := aX   
'Sends aXhold array to temporary sort array. 
  Sort[0]  := aXhold[0] 
  Sort[1]  := aXhold[1] 
  Sort[2]  := aXhold[2] 
  Sort[3]  := aXhold[3] 
  Sort[4]  := aXhold[4] 
  Sort[5]  := aXhold[5] 
  Sort[6]  := aXhold[6]
  Sort[7]  := aXhold[7] 
  Sort[8]  := aXhold[8] 
  Sort[9]  := aXhold[9] 
  Sort[10] := aXhold[10]
'Sorts the temporary array.  
  pasmshellsort(@Sort,11,ASC)
  
  aX := Sort[5]
  Debughold[3] := aX

'*********************************************
'Disgards oldest aY value and reorders hold array.  
  aYhold[10] := aYhold[9]
  aYhold[9]  := aYhold[8]
  aYhold[8]  := aYhold[7]
  aYhold[7]  := aYhold[6]
  aYhold[6]  := aYhold[5]
  aYhold[5]  := aYhold[4]
  aYhold[4]  := aYhold[3]
  aYhold[3]  := aYhold[2]
  aYhold[2]  := aYhold[1]
  aYhold[1]  := aYhold[0] 
  aYhold[0]  := aY   
'Sends aYhold array to temporary sort array. 
  Sort[0]  := aYhold[0] 
  Sort[1]  := aYhold[1] 
  Sort[2]  := aYhold[2] 
  Sort[3]  := aYhold[3] 
  Sort[4]  := aYhold[4] 
  Sort[5]  := aYhold[5] 
  Sort[6]  := aYhold[6]
  Sort[7]  := aYhold[7] 
  Sort[8]  := aYhold[8] 
  Sort[9]  := aYhold[9] 
  Sort[10] := aYhold[10]
  
  pasmshellsort(@Sort,11,ASC)
  
  aY := Sort[5]
  Debughold[4] := aY
  
'*********************************************
'Disgards oldest aZ value and reorders hold array.   
  aZhold[10] := aZhold[9]
  aZhold[9]  := aZhold[8]
  aZhold[8]  := aZhold[7]
  aZhold[7]  := aZhold[6]
  aZhold[6]  := aZhold[5]
  aZhold[5]  := aZhold[4]
  aZhold[4]  := aZhold[3]
  aZhold[3]  := aZhold[2]
  aZhold[2]  := aZhold[1]
  aZhold[1]  := aZhold[0] 
  aZhold[0]  := aZ   
'Sends aZhold array to temporary sort array. 
  Sort[0]  := aZhold[0] 
  Sort[1]  := aZhold[1] 
  Sort[2]  := aZhold[2] 
  Sort[3]  := aZhold[3] 
  Sort[4]  := aZhold[4] 
  Sort[5]  := aZhold[5] 
  Sort[6]  := aZhold[6]
  Sort[7]  := aZhold[7] 
  Sort[8]  := aZhold[8] 
  Sort[9]  := aZhold[9] 
  Sort[10] := aZhold[10]
'Sorts the temporary array.  
  pasmshellsort(@Sort,11,ASC)
  
  aZ := Sort[5]
  Debughold[5] := aZ
  elapsed += cnt - 544          'Used to time sorts.
  elapsed := elapsed/uS         'Used to time sorts.
'*********************************************   
PUB pasmshellsort (arrayAddr, arraylength, asc_desc) : done
'Fires up pasm shell sort and sorts temp array.
  parAddr := arrayAddr
  parlen := arraylength
  ascdesc := asc_desc 
  done := 0
  
  IF (cognew(@shellsrt, @done) => 0)
    REPEAT UNTIL (done)

'*********************************************
'Starts display cog for PST
PUB Initiate_Display_Cog : Status

  Status := Cog := cognew(Display, @DisplayStack[0])    'Starts Pub Display in a new Cog.

'********************************************* 
PUB Display

  PST.Clear

  Repeat
    PST.Clear
    PST.Str(String("Raw ACC = "))
    PST.Dec(Debughold[0])
    PST.Str(String(" , "))
    PST.Dec(Debughold[1])
    PST.Str(String(" , "))
    PST.Dec(Debughold[2])

    PST.NewLine
    PST.Str(String("Median ACC = "))
    PST.Dec(Debughold[3])
    PST.Str(String(" , "))
    PST.Dec(Debughold[4])
    PST.Str(String(" , "))
    PST.Dec(Debughold[5])

    PST.NewLine
    PST.Dec(elapsed)
    waitcnt(clkfreq/10 + cnt)

'*********************************************        
DAT

                        ORG
shellsrt
                        MOV     parAddr2, parAddr

                        MOV     pinc, parlen                                    ' inc := arraylength
                        SHR     pinc, #1                                        ' arraylength >> 1 

bigloop                 TJZ     pinc, #end                                      ' REPEAT WHILE (inc > 0)
                        MOV     idx, pinc                                       ' REPEAT i FROM inc
frompinc                CMP     idx, parlen     WZ, WC                          ' TO arraylength
              IF_AE     JMP     #lfrompinc
                                    
                        MOV     parAddr, parAddr2                               ' arrayAddr
                        MOV     addrAdd, idx                                    ' i
                        SHL     addrAdd, #2                                     ' [i]
                        ADD     parAddr, addrAdd                                ' long[arrayAddr][i]
                        RDLONG  pval, parAddr                                   ' val := long[arrayAddr][i]

                        MOV     jdx, idx                                        ' j := i

innerloop               CMP     jdx, pinc       WZ, WC                          ' REPEAT WHILE (j => inc
              IF_B      JMP     #linnerloop

                        MOV     addr, parAddr2                                  ' arrayAddr
                        MOV     addrAdd, jdx                                    ' j
                        SUB     addrAdd, pinc                                   ' j - inc                 
                        SHL     addrAdd, #2                                     ' [j - inc]
                        ADD     addr, addrAdd                                   ' long[arrayAddr][j - inc]
                        RDLONG  p1, addr                                        ' long[arrayAddr][j - inc]

                        CMPS    p1, pval        WZ, WC
              IF_Z      JMP     #linnerloop          
                        CMP     ascdesc, #0     WZ                          
             IF_Z_AND_C JMP     #linnerloop                                     ' long[arrayAddr][j - inc] > val
              IF_A      JMP     #linnerloop                                     ' long[arrayAddr][j - inc] < val   (IF_A == IF_NZ_AND_NC)

                        MOV     parAddr, parAddr2                               ' arrayAddr
                        MOV     addrAdd, jdx                                    ' j
                        SHL     addrAdd, #2                                     ' [j]
                        ADD     parAddr, addrAdd                                ' long[arrayAddr][j]
                        WRLONG  p1, parAddr                                     ' long[arrayAddr][j] := long[arrayAddr][j - inc]
                        SUBS    jdx, pinc                                       ' j -= inc
                        JMP     #innerloop

linnerloop              MOV     parAddr, parAddr2                               ' arrayAddr
                        MOV     addrAdd, jdx                                    ' j
                        SHL     addrAdd, #2                                     ' [j]
                        ADD     parAddr, addrAdd                                ' long[arrayAddr][j]
                        WRLONG  pval, parAddr                                   ' long[arrayAddr][j] := val
                        ADD     idx, #1                                         ' STEP 1
                        JMP     #frompinc

lfrompinc               SHR     pinc, #1                                        ' inc >>= 1
                        JMP     #bigloop

end                     WRLONG  negone, PAR
                        COGID   p1                                              ' get cog id
                        COGSTOP p1                                              ' kill this cog


negone                  LONG    -1

parAddr                 LONG    0
parlen                  LONG    0
ascdesc                 LONG    0

parAddr2                RES
addrAdd                 RES
addr                    RES
pinc                    RES
idx                     RES
jdx                     RES
pval                    RES

p1                      RES 

                        FIT